package CH.ifa.draw.io.exportFormats;

import CH.ifa.draw.framework.Drawing;

import CH.ifa.draw.io.PSFileFilter;

import CH.ifa.draw.util.ColorMap;
import CH.ifa.draw.util.PostscriptWriter;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Rectangle;

import java.io.File;
import java.io.FileOutputStream;


public class PSExportFormat extends ExportFormatAbstract {
    public static org.apache.log4j.Logger logger = org.apache.log4j.Logger
                                                   .getLogger(PSExportFormat.class);

    // Attributes
    // Construktor
    public PSExportFormat() {
        super("PostScript", new PSFileFilter());
    }

    // Methods


    /**
     * @see de.renew.io.ExportFormat#canExportNto1()
     */
    public boolean canExportNto1() {
        return false;
    }

    /**
     * @see de.renew.io.ExportFormat#export(CH.ifa.draw.framework.Drawing, java.net.URI)
     */
    public File export(Drawing drawing, File path) throws Exception {
        File result = null;
        if (drawing != null && path != null) {
            result = path;
            FileOutputStream stream = new FileOutputStream(result);
            PostscriptWriter ps = null;
            Dimension pageSize = new Dimension(535, 788);
            Rectangle drawingBox = drawing.getBounds();
            ps = new PostscriptWriter(stream);
            ps.translate(32, -(32 + pageSize.height));
            pageSize.width = Math.max(pageSize.width,
                                      drawingBox.x + drawingBox.width);
            pageSize.height = Math.max(pageSize.height,
                                       drawingBox.y + drawingBox.height);


            // ps.translate(-32,-32+pageSize.height);
            // ps.setClip(64,64,pageSize.width-128,pageSize.height-128);
            // replaced by:
            ps.setClip(0, 0, pageSize.width, pageSize.height);
            double scale = Math.min(535.0 / pageSize.width,
                                    778.0 / pageSize.height);

            if (scale < 1.0) {
                logger.debug("Scaling PS output by " + scale);
                ps.scale(scale, scale);
            }
            ps.setColorConversion(ColorMap.BACKGROUND, Color.white);
            drawing.draw(ps);

            // view().drawHandles(ps);
            ps.dispose();
        }
        assert (result != null) : "Failure in PSExportFormat: result == null";
        return result;
    }

    /**
     * @see de.renew.io.ExportFormat#export(CH.ifa.draw.framework.Drawing[], java.net.URI)
     */
    public File export(Drawing[] drawings, File path) throws Exception {
        File result = null;
        assert (result != null) : "Failure in EPSExportFormat: result == null";
        return result;
    }

    public boolean canExportDrawing(Drawing drawing) {
        boolean result = false;
        result = true;
        return result;
    }
}