package de.renew.plugin.command;

import de.renew.plugin.IPlugin;
import de.renew.plugin.PluginManager;
import de.renew.plugin.PluginProperties;

import java.io.PrintStream;

import java.util.Iterator;


/**
 * This command is used to query a user property value as returned
 * by the PluginProperties.getUserProperties() static method.
 *
 * @author J&ouml;rn Schumacher
 */
public class GetPropertyCommand implements CLCommand {

    /**
     * Prints the value of a property as it is known to any plugin.
     * The property is specified via <code>args</code>, the output goes to
     * <code>response</code>.
     * @param args {@inheritDoc}
     * @param response {@inheritDoc}
     **/
    public void execute(String[] args, PrintStream response) {
        for (int i = 0; i < args.length; i++) {
            getProperty(args[i], response);
        }
    }

    protected void getProperty(String arg, PrintStream response) {
        String globalValue = PluginProperties.getUserProperties().getProperty(arg);
        if (globalValue != null) {
            response.println(arg + " = " + globalValue
                             + " (global value of property)");
        }
        boolean noMatches = true;
        Iterator plugins = PluginManager.getInstance().getPlugins().iterator();
        while (plugins.hasNext()) {
            IPlugin p = (IPlugin) plugins.next();
            PluginProperties props = p.getProperties();
            String localValue = props.getProperty(arg);
            if (props.isKnownProperty(arg)) {
                if (localValue == null) {
                    response.println("Property " + arg + " is known to plugin "
                                     + p.getName() + ", but not set.");
                } else {
                    response.println(arg + " = " + localValue
                                     + " (known property in plugin "
                                     + p.getName() + ")");
                }
                noMatches = false;
            } else if (!(localValue == globalValue
                               || (localValue != null
                               && localValue.equals(globalValue)))) {
                if (localValue == null) {
                    response.println("Property " + arg
                                     + " is not set locally in plugin "
                                     + p.getName() + ".");
                } else {
                    response.println(arg + " = " + localValue
                                     + " (local value in plugin " + p.getName()
                                     + ")");
                }
                noMatches = false;
            }
        }
        if (globalValue == null && noMatches) {
            response.println("Property " + arg + " is not set.");
        }
    }

    /**
     * @see de.renew.plugin.command.CLCommand#getDescription()
     */
    public String getDescription() {
        return "returns the value of the given PluginProperty";
    }
}