package de.renew.plugin.command;

import de.renew.plugin.CollectionLister;
import de.renew.plugin.IPlugin;
import de.renew.plugin.PluginAdapter;
import de.renew.plugin.PluginManager;

import java.io.PrintStream;

import java.util.Iterator;


/**
 * This command displays a list of the previously loaded plugins in the system.
 *
 * @author J&ouml;rn Schumacher
 *
 */
public class ListCommand implements CLCommand {
    public static org.apache.log4j.Logger logger = org.apache.log4j.Logger
                                                   .getLogger(InfoCommand.class);

    /**
     * Prints the list of loaded plugins to the <code>response</code> stream.
     *
     * @param args
     *            {@inheritDoc}
     * @param response
     *            {@inheritDoc}
     */
    public void execute(String[] args, PrintStream response) {
        // get Plugin name from input
        String pluginName = CollectionLister.toString(args, " ");
        if (args.length > 0) {
            if ("-l".equals(args[0]) || "l".equals(args[0])
                        || "--long".equals(args[0]) || "long".equals(args[0])) {
                try {
                    Iterator it = PluginManager.getInstance().getPlugins()
                                               .iterator();

                    while (it.hasNext()) {
                        IPlugin plugin = (IPlugin) it.next();
                        String versionstring = "";
                        if (plugin instanceof PluginAdapter) {
                            PluginAdapter pa = (PluginAdapter) plugin;
                            versionstring = pa.getVersion();
                        }
                        response.println(plugin.getProperties()
                                               .getFilteredProperty(".date")
                                         + "\t "
                                         + plugin.getProperties()
                                                 .getFilteredProperty(".user")
                                         + "\t " + versionstring + "\t "
                                         + plugin.getName());
                    }
                } catch (ArrayIndexOutOfBoundsException e) {
                } catch (NumberFormatException e) {
                    response.println("cleanup canceled.");
                } catch (Exception e) {
                    response.println("cleanup canceled: " + e + "; "
                                     + e.getMessage());
                    logger.error(e.getMessage(), e);
                }
            } else if ("-c".equals(args[0]) || "c".equals(args[0])
                               || "--comment".equals(args[0])
                               || "comment".equals(args[0])) {
                try {
                    Iterator it = PluginManager.getInstance().getPlugins()
                                               .iterator();

                    while (it.hasNext()) {
                        IPlugin plugin = (IPlugin) it.next();
                        String versionstring = "";
                        if (plugin instanceof PluginAdapter) {
                            PluginAdapter pa = (PluginAdapter) plugin;
                            versionstring = pa.getVersion();
                        }
                        String propvalue = plugin.getProperties()
                                                 .getFilteredProperty("comment");
                        if (!"No comment given in etc/comment.txt".equals(propvalue)) {
                            response.println(plugin.getName() + "\t "
                                             + propvalue);

                        }
                    }
                } catch (ArrayIndexOutOfBoundsException e) {
                } catch (NumberFormatException e) {
                    response.println("cleanup canceled.");
                } catch (Exception e) {
                    response.println("cleanup canceled: " + e + "; "
                                     + e.getMessage());
                    logger.error(e.getMessage(), e);
                }
            }
            return;
        }

        Iterator it = PluginManager.getInstance().getPlugins().iterator();
        while (it.hasNext()) {
            response.println(it.next());
        }
    }

    public String getDescription() {
        return "lists all loaded plugins. Options: [-l| --long] for more information; [-c| --comment] to show compile comments.";
    }
}